from .google_analytics import (
    GAEventsDataFrameType,
    GASessionsDataFrameType,
    GAUsersDataFrameType,
)


__all__ = [
    "GAEventsDataFrameType",
    "GASessionsDataFrameType",
    "GAUsersDataFrameType",
]



from datetime import datetime
import re

from apiclient.discovery import build
import httplib2
from oauth2client.service_account import ServiceAccountCredentials
from pandas import DataFrame


CAMEL_CASE_TO_SNAKE_CASE = r"(?<!^)(?=[A-Z])"


def normalize_header(value):
    return re.sub(
        CAMEL_CASE_TO_SNAKE_CASE, "_", value.replace("ga:", "")
    ).lower()


def connect_to_google_analytics(sa_credentials):
    credentials = ServiceAccountCredentials.from_json_keyfile_dict(
        sa_credentials,
        ["https://www.googleapis.com/auth/analytics.readonly"],
    )

    # Create a service object
    http = credentials.refresh(httplib2.Http())
    discovery_url = "https://analyticsreporting.googleapis.com/$discovery/rest"

    service = build(
        "analytics",
        "v4",
        http=http,
        discoveryServiceUrl=discovery_url,
    )

    return service


def extract(response):
    reports = response["reports"][0]

    # fmt: off
    columns = [
        normalize_header(dimension)
        for dimension in reports["columnHeader"]["dimensions"]
    ] + [
        normalize_header(metric["name"])
        for metric
        in reports["columnHeader"]["metricHeader"]["metricHeaderEntries"]
    ]
    # fmt: on

    rows = []
    try:
        for row in reports["data"]["rows"]:
            metrics = [
                value for metric in row["metrics"] for value in metric["values"]
            ]
            rows.append(row["dimensions"] + metrics)

    except KeyError:
        rows = []

    dataframe = DataFrame(columns=columns, data=rows)
    return dataframe


def prepare_body(view_id, date, metrics, dimensions):
    return {
        "reportRequests": [
            {
                "viewId": view_id,
                "dateRanges": [
                    {
                        "startDate": date,
                        "endDate": date,
                    }
                ],
                "metrics": [
                    {"expression": "ga:{}".format(metric)} for metric in metrics
                ],
                "dimensions": [
                    {"name": "ga:{}".format(dimension)}
                    for dimension in dimensions
                ],
                "pageSize": 1000,
            }
        ]
    }


def ga_events(
    sa_credentials,
    customer_name,
    customer_key,
    view_id,
    partition_key,
):
    service = connect_to_google_analytics(sa_credentials)

    response = (
        service.reports()
        .batchGet(
            body=prepare_body(
                view_id,
                partition_key,
                ["totalEvents", "uniqueEvents", "eventValue"],
                ["eventCategory", "eventAction", "eventLabel"],
            )
        )
        .execute()
    )

    dataframe = extract(response)
    dataframe_length = len(dataframe.index)
    view_id = int(view_id)
    partition_key_datetime = datetime.strptime(partition_key, "%Y-%m-%d")

    # fmt: off
    dataframe["event_category"] = dataframe["event_category"].astype(str)
    dataframe["event_action"] = dataframe["event_action"].astype(str)
    dataframe["event_label"] = dataframe["event_label"].astype(str)
    dataframe["total_events"] = dataframe["total_events"].astype(int)
    dataframe["unique_events"] = dataframe["unique_events"].astype(int)
    dataframe["event_value"] = dataframe["event_value"].astype(int)

    dataframe["customer_key"] = [customer_key] * dataframe_length
    dataframe["customer_name"] = [customer_name] * dataframe_length
    dataframe["view_id"] = [view_id] * dataframe_length
    dataframe["partition_key"] = [partition_key_datetime] * dataframe_length
    # fmt: on
    return dataframe


def ga_sessions(
    sa_credentials,
    customer_name,
    customer_key,
    view_id,
    partition_key,
):
    service = connect_to_google_analytics(sa_credentials)

    response = (
        service.reports()
        .batchGet(
            body=prepare_body(
                view_id,
                partition_key,
                [
                    "sessions",
                    "bounces",
                    "sessionsPerUser",
                    "sessionDuration",
                    "pageLoadTime",
                ],
                [
                    "continent",
                    "country",
                    "countryIsoCode",
                    "region",
                    "regionIsoCode",
                    "city",
                    "pagePath",
                ],
            )
        )
        .execute()
    )

    dataframe = extract(response)
    dataframe_length = len(dataframe.index)
    view_id = int(view_id)
    partition_key_datetime = datetime.strptime(partition_key, "%Y-%m-%d")

    # fmt: off
    dataframe["continent"] = dataframe["continent"].astype(str)
    dataframe["country"] = dataframe["country"].astype(str)
    dataframe["country_iso_code"] = dataframe["country_iso_code"].astype(str)
    dataframe["region"] = dataframe["region"].astype(str)
    dataframe["region_iso_code"] = dataframe["region_iso_code"].astype(str)
    dataframe["city"] = dataframe["city"].astype(str)
    dataframe["page_path"] = dataframe["page_path"].astype(str)
    dataframe["sessions"] = dataframe["sessions"].astype(int)
    dataframe["bounces"] = dataframe["bounces"].astype(int)
    dataframe["sessions_per_user"] = (
        dataframe["sessions_per_user"].astype(float)
    )
    dataframe["session_duration"] = (
        dataframe["session_duration"].astype(float)
    )
    dataframe["page_load_time"] = dataframe["page_load_time"].astype(int)

    dataframe["customer_key"] = [customer_key] * dataframe_length
    dataframe["customer_name"] = [customer_name] * dataframe_length
    dataframe["view_id"] = [view_id] * dataframe_length
    dataframe["partition_key"] = [partition_key_datetime] * dataframe_length
    # fmt: on
    return dataframe


def ga_users(
    sa_credentials,
    customer_name,
    customer_key,
    view_id,
    partition_key,
):
    service = connect_to_google_analytics(sa_credentials)

    response = (
        service.reports()
        .batchGet(
            body=prepare_body(
                view_id,
                partition_key,
                [
                    "users",
                    "newUsers",
                    "percentNewSessions",
                    "sessionsPerUser",
                    "sessionDuration",
                    "pageLoadTime",
                ],
                [
                    "continent",
                    "country",
                    "countryIsoCode",
                    "region",
                    "regionIsoCode",
                    "city",
                    "pagePath",
                ],
            )
        )
        .execute()
    )

    dataframe = extract(response)
    dataframe_length = len(dataframe.index)
    view_id = int(view_id)
    partition_key_datetime = datetime.strptime(partition_key, "%Y-%m-%d")

    # fmt: off
    dataframe["continent"] = dataframe["continent"].astype(str)
    dataframe["country"] = dataframe["country"].astype(str)
    dataframe["country_iso_code"] = dataframe["country_iso_code"].astype(str)
    dataframe["region"] = dataframe["region"].astype(str)
    dataframe["region_iso_code"] = dataframe["region_iso_code"].astype(str)
    dataframe["city"] = dataframe["city"].astype(str)
    dataframe["page_path"] = dataframe["page_path"].astype(str)
    dataframe["users"] = dataframe["users"].astype(int)
    dataframe["new_users"] = dataframe["new_users"].astype(int)
    dataframe["percent_new_sessions"] = (
        dataframe["percent_new_sessions"].astype(float)
    )
    dataframe["sessions_per_user"] = (
        dataframe["sessions_per_user"].astype(float)
    )
    dataframe["session_duration"] = dataframe["session_duration"].astype(float)
    dataframe["page_load_time"] = dataframe["page_load_time"].astype(int)

    dataframe["customer_key"] = [customer_key] * dataframe_length
    dataframe["customer_name"] = [customer_name] * dataframe_length
    dataframe["view_id"] = [view_id] * dataframe_length
    dataframe["partition_key"] = [partition_key_datetime] * dataframe_length
    # fmt: on
    return dataframe


def ga_events_dataframe(context, secret_path):
    secrets = context.resources.vault.get_data(secret_path)

    view_id = secrets["view_id"]
    sa_credentials = secrets["sa_credentials"]
    customer_name = secrets["customer_name"]
    partition_key = context.asset_partition_key_for_output()
    customer_key = secret_path.replace("analytics/google-analytics/", "")

    dataframe = ga_events(
        sa_credentials, customer_name, customer_key, view_id, partition_key
    )

    metadata = {
        "table": "ga_events",
        "partition_expr": "partition_key,customer_key",
        "partition_key": partition_key,
        "customer_key": customer_key,
        "view_id": view_id,
        "customer_name": customer_name,
    }

    if isinstance(dataframe, DataFrame):
        metadata["num_rows"] = len(dataframe.index)

    return dataframe, metadata


def ga_sessions_dataframe(context, secret_path):
    secrets = context.resources.vault.get_data(secret_path)

    view_id = secrets["view_id"]
    sa_credentials = secrets["sa_credentials"]
    customer_name = secrets["customer_name"]
    partition_key = context.asset_partition_key_for_output()
    customer_key = secret_path.replace("analytics/google-analytics/", "")

    dataframe = ga_sessions(
        sa_credentials, customer_name, customer_key, view_id, partition_key
    )

    metadata = {
        "table": "ga_sessions",
        "partition_expr": "partition_key,customer_key",
        "partition_key": partition_key,
        "customer_key": customer_key,
        "view_id": view_id,
        "customer_name": customer_name,
    }

    if isinstance(dataframe, DataFrame):
        metadata["num_rows"] = len(dataframe.index)

    return dataframe, metadata


def ga_users_dataframe(context, secret_path):
    secrets = context.resources.vault.get_data(secret_path)

    view_id = secrets["view_id"]
    sa_credentials = secrets["sa_credentials"]
    customer_name = secrets["customer_name"]
    partition_key = context.asset_partition_key_for_output()
    customer_key = secret_path.replace("analytics/google-analytics/", "")

    dataframe = ga_users(
        sa_credentials, customer_name, customer_key, view_id, partition_key
    )

    metadata = {
        "table": "ga_users",
        "partition_expr": "partition_key,customer_key",
        "partition_key": partition_key,
        "customer_key": customer_key,
        "view_id": view_id,
        "customer_name": customer_name,
    }

    if isinstance(dataframe, DataFrame):
        metadata["num_rows"] = len(dataframe.index)

    return dataframe, metadata



from .abstract import ObjectStorage
from .csv_partitioned_io_manager import csv_partitioned_io_manager
from .json_partitioned_io_manager import json_partitioned_io_manager
from .postgres_partitioned_io_manager import postgres_partitioned_io_manager
from .postgres_io_manager import postgres_io_manager
from .vault import vault


__all__ = [
    # Abstract
    "ObjectStorage",
    # IO Managers
    "csv_partitioned_io_manager",
    "json_partitioned_io_manager",
    "postgres_partitioned_io_manager",
    "postgres_io_manager",
    # Vault
    "vault",
]



from .container_codes import (
    ContainerCodeDataFrame,
    ContainerGroupDataFrame,
)
from .continent_codes import ContinentCodeDataFrame
from .country_codes import CountryCodeDataFrame
from .currency_codes import CurrencyCodeDataFrame
from .language_codes import LanguageCodeDataFrame

# from .subdivision_codes import SubDivisionCodeDataFrame
# from .units_of_measure import (
#     UnitsOfMeasureDataFrame,
#     UnitsOfMeasureLevelDataFrame,
# )


__all__ = [
    "ContainerCodeDataFrame",
    "ContainerGroupDataFrame",
    "ContinentCodeDataFrame",
    "CountryCodeDataFrame",
    "CurrencyCodeDataFrame",
    "LanguageCodeDataFrame",
    # "UnitsOfMeasureDataFrame",
    # "UnitsOfMeasureLevelDataFrame",
    # "SubDivisionCodeDataFrame",
]



from dagster_pandas import create_dagster_pandas_dataframe_type, PandasColumn

RegionDataFrame = create_dagster_pandas_dataframe_type(
    name="RegionDataFrame",
    columns=[
        PandasColumn.datetime_column(
            "partition_key",
            is_required=True,
        ),
        PandasColumn.string_column("name"),
        PandasColumn.string_column("slug"),
        PandasColumn.string_column("description"),
        PandasColumn.string_column("country_iso_code"),
        PandasColumn.string_column("iso_code"),
    ],
)



import re

from slugify import slugify


CAMEL_CASE_TO_SNAKE_CASE = r"(?<!^)(?=[A-Z])"


def normalize_header(value):
    """
    columns = [text.normalize_header(column) for column in dataframe.columns]
    dataframe.columns = columns
    """
    value = re.sub(CAMEL_CASE_TO_SNAKE_CASE, "_", value).lower()
    value = slugify(value, separator="_")
    return value


def snakify(value):
    return slugify(value, separator="_").lower()


def remove_header_infixes(value, infexes):
    """
    infexes = ["ab:", "cd:", "ef:",]
    columns = [
        text.remove_header_infixes(column, infexes)
        for column in dataframe.columns
    ]
    dataframe.columns = columns
    """
    for infix in infexes:
        value = value.replace(infix, "")



from logging import getLogger
from os import environ

import hvac

from dagster import resource


LOGGER = getLogger(__name__)


class Vault:
    client = None

    hostname = None
    token = None

    def __init__(self):
        self.hostname = environ.get("VAULT_ADDR")
        self.token = environ.get("VAULT_TOKEN")
        self.keys = environ.get("VAULT_KEYS").split(",")

        self.client = hvac.Client(
            url=self.hostname, token=self.token, verify=False
        )

        if not self.client.sys.is_initialized():
            raise Exception("Vault not initialized")

        if self.client.sys.is_sealed():
            self.client.sys.submit_unseal_keys(self.keys)

    def get_data(self, path):
        response = self.client.secrets.kv.read_secret_version(path=path)
        return response["data"]["data"]


@resource
def vault(init_context):
    return Vault()



from dagster_pandas import create_dagster_pandas_dataframe_type, PandasColumn

ContinentCodeDataFrame = create_dagster_pandas_dataframe_type(
    name="ContinentCodeDataFrame",
    columns=[
        PandasColumn.datetime_column(
            "partition_key",
            is_required=True,
        ),
        PandasColumn.string_column(
            "code",
        ),
        PandasColumn.string_column(
            "name",
        ),
    ],
)



import pandera
from pandera.typing import Series, String, DateTime

from dagster_pandera import pandera_schema_to_dagster_type


# fmt: off
class SubDivisionCodesDataFrame(pandera.SchemaModel):
    partition_key: Series[DateTime] = pandera.Field(
        nullable=True,
        description="Current partition.",
    )
    country: Series[String] = pandera.Field(
        nullable=True,
        description="Country ISO / Identification Code.",
    )
    code: Series[String] = pandera.Field(
        nullable=True,
        description="Sub Division UN Identification Code.",
    )
    name: Series[String] = pandera.Field(
        nullable=True,
        description="Sub Division name.",
    )


SubDivisionCodesDataFrameType = pandera_schema_to_dagster_type(
    SubDivisionCodesDataFrame
)
# fmt: on



from dagster_pandas import create_dagster_pandas_dataframe_type, PandasColumn

CountryCodeDataFrame = create_dagster_pandas_dataframe_type(
    name="CountryCodeDataFrame",
    columns=[
        PandasColumn.datetime_column(
            "partition_key",
            is_required=True,
        ),
        PandasColumn.string_column(
            "fifa",
        ),
        PandasColumn.string_column(
            "dial",
        ),
        PandasColumn.string_column(
            "iso3166_1_alpha_3",
        ),
        PandasColumn.string_column(
            "marc",
        ),
        PandasColumn.string_column(
            "is_independent",
        ),
        PandasColumn.string_column(
            "iso3166_1_numeric",
        ),
        PandasColumn.string_column(
            "gaul",
        ),
        PandasColumn.string_column(
            "fips",
        ),
        PandasColumn.string_column(
            "wmo",
        ),
        PandasColumn.string_column(
            "iso3166_1_alpha_2",
        ),
        PandasColumn.string_column(
            "itu",
        ),
        PandasColumn.string_column(
            "ioc",
        ),
        PandasColumn.string_column(
            "ds",
        ),
        PandasColumn.string_column(
            "unterm_spanish_formal",
        ),
        PandasColumn.string_column(
            "global_code",
        ),
        PandasColumn.string_column(
            "intermediate_region_code",
        ),
        PandasColumn.string_column(
            "official_name_fr",
        ),
        PandasColumn.string_column(
            "unterm_french_short",
        ),
        PandasColumn.string_column(
            "iso4217_currency_name",
        ),
        PandasColumn.string_column(
            "developed_developing_countries",
        ),
        PandasColumn.string_column(
            "unterm_russian_formal",
        ),
        PandasColumn.string_column(
            "unterm_english_short",
        ),
        PandasColumn.string_column(
            "iso4217_currency_alphabetic_code",
        ),
        PandasColumn.string_column(
            "small_island_developing_states",
        ),
        PandasColumn.string_column(
            "unterm_spanish_short",
        ),
        PandasColumn.string_column(
            "iso4217_currency_numeric_code",
        ),
        PandasColumn.string_column(
            "unterm_chinese_formal",
        ),
        PandasColumn.string_column(
            "unterm_french_formal",
        ),
        PandasColumn.string_column(
            "unterm_russian_short",
        ),
        PandasColumn.string_column(
            "m49",
        ),
        PandasColumn.string_column(
            "sub_region_code",
        ),
        PandasColumn.string_column(
            "region_code",
        ),
        PandasColumn.string_column(
            "official_name_ar",
        ),
        PandasColumn.string_column(
            "iso4217_currency_minor_unit",
        ),
        PandasColumn.string_column(
            "unterm_arabic_formal",
        ),
        PandasColumn.string_column(
            "unterm_chinese_short",
        ),
        PandasColumn.string_column(
            "land_locked_developing_countries",
        ),
        PandasColumn.string_column(
            "intermediate_region_name",
        ),
        PandasColumn.string_column(
            "official_name_es",
        ),
        PandasColumn.string_column(
            "unterm_english_formal",
        ),
        PandasColumn.string_column(
            "official_name_cn",
        ),
        PandasColumn.string_column(
            "official_name_en",
        ),
        PandasColumn.string_column(
            "iso4217_currency_country_name",
        ),
        PandasColumn.string_column(
            "least_developed_countries",
        ),
        PandasColumn.string_column(
            "region_name",
        ),
        PandasColumn.string_column(
            "unterm_arabic_short",
        ),
        PandasColumn.string_column(
            "sub_region_name",
        ),
        PandasColumn.string_column(
            "official_name_ru",
        ),
        PandasColumn.string_column(
            "global_name",
        ),
        PandasColumn.string_column(
            "capital",
        ),
        PandasColumn.string_column(
            "continent",
        ),
        PandasColumn.string_column(
            "tld",
        ),
        PandasColumn.string_column(
            "languages",
        ),
        PandasColumn.string_column(
            "geoname_id",
        ),
        PandasColumn.string_column(
            "cldr_display_name",
        ),
        PandasColumn.string_column(
            "edgar",
        ),
    ],
)



from dagster_pandas import create_dagster_pandas_dataframe_type, PandasColumn

PostalCodeDataFrame = create_dagster_pandas_dataframe_type(
    name="PostalCodeDataFrame",
    columns=[
        PandasColumn.datetime_column(
            "partition_key",
            is_required=True,
        ),
        PandasColumn.string_column(
            "country_code",
            # iso country code, 2 characters
        ),
        PandasColumn.string_column(
            "postal_code",
            # Postal code
        ),
        PandasColumn.string_column(
            "name",
            # Place name
        ),
        PandasColumn.string_column(
            "admin_name_1",
            # 1. order subdivision (state) varchar(100)
        ),
        PandasColumn.string_column(
            "admin_code_1",
            # 1. order subdivision (state) varchar(20)
        ),
        PandasColumn.string_column(
            "admin_name_2",
            # 2. order subdivision (county/province) varchar(100)
        ),
        PandasColumn.string_column(
            "admin_code_2",
            # 2. order subdivision (county/province) varchar(20)
        ),
        PandasColumn.string_column(
            "admin_name_3",
            # 3. order subdivision (community) varchar(100)
        ),
        PandasColumn.string_column(
            "admin_code_3",
            # 3. order subdivision (community) varchar(20)
        ),
        PandasColumn.float_column(
            "latitude",
            # estimated latitude (wgs84)
        ),
        PandasColumn.float_column(
            "longitude",
            # estimated longitude (wgs84)
        ),
        PandasColumn.float_column(
            "accuracy",
            # accuracy of lat/lng from 1 = estimated, 4 = geonameid,
            # 6 = centroid of addresses or shape
        ),
    ],
)



from .common import (
    ContinentDataFrame,
    CountryDataFrame,
    PostCodeDataFrame,
    RegionDataFrame,
    SubRegionDataFrame,
)
from .datahub import (
    ContainerCodeDataFrame,
    ContainerGroupDataFrame,
    ContinentCodeDataFrame,
    CountryCodeDataFrame,
    CurrencyCodeDataFrame,
    LanguageCodeDataFrame,
)
from .geonames import PostalCodeDataFrame
from .google_analytics import (
    GAEventsDataFrameType,
    GASessionsDataFrameType,
    GAUsersDataFrameType,
)
from .open_data_fr import (
    EstablishmentsDataFrame,
)


__all__ = [
    "ContinentDataFrame",
    "CountryDataFrame",
    "PostCodeDataFrame",
    "RegionDataFrame",
    "SubRegionDataFrame",
    "ContainerCodeDataFrame",
    "ContainerGroupDataFrame",
    "ContinentCodeDataFrame",
    "CountryCodeDataFrame",
    "CurrencyCodeDataFrame",
    "LanguageCodeDataFrame",
    "PostalCodeDataFrame",
    "GAEventsDataFrameType",
    "GASessionsDataFrameType",
    "GAUsersDataFrameType",
    "EstablishmentsDataFrame",
]



from setuptools import setup, find_packages

setup(
    name="dagster-toolbox",
    version="0.0.15",
    packages=find_packages(),
    author="Nicolas RAMY",
    author_email="nicolas.ramy@darkelda.com",
    license="MIT",
    description="A set of tools to ease Dagster usage",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "dagster==1.6.13",
        "dagster-aws==0.22.13",
        "dagster-pandas==0.22.13",
        "dagster-pandera==0.22.13",
        "dagster-postgres==0.22.13",
        "google-api-python-client==2.123.0",
        "hvac==2.1.0",
        "minio==7.2.5",
        "oauth2client==4.1.3",
        "pandas==2.2.1",
        "python-slugify==8.0.4",
        "requests==2.31.0",
        "SQLAlchemy==2.0.29",
    ],
    url="https://nicolasramy.github.io/dagster-toolbox/",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.10",
        "Topic :: Terminals",
        "Topic :: Utilities",
    ],
)



from datetime import datetime
from hashlib import sha512
import stat

from slugify import slugify

from .partitions import get_current_partition


def prepare_object(file_resource, partitioned):
    file_stat = file_resource.stat()
    file_last_access = file_stat.st_atime
    file_last_modification = file_stat.st_mtime
    file_size = file_stat.st_size

    if partitioned:
        partition_key = get_current_partition()

    else:
        partition_key = datetime.fromtimestamp(file_last_modification).strftime(
            "%Y-%m-%d"
        )

    metadata = {
        "last_access": file_last_access,
        "last_modification": file_last_modification,
        "size": file_size,
        "partition": partition_key,
    }

    return partition_key, metadata


# fmt: off
def sftp_get_recursive(
    context,
    sftp_client,
    path,
    destination,
    partitioned,
    files,
):
    context.log.debug(f"Open {path}")
    for item in sftp_client.listdir_attr(path):
        remote_path = f"{path}/{item.filename}"
        mode = item.st_mode

        if stat.S_ISDIR(mode):
            sftp_get_recursive(
                context,
                sftp_client,
                remote_path,
                destination,
                partitioned,
                files,
            )

        elif stat.S_ISREG(mode):
            with sftp_client.file(remote_path, mode="r") as file_resource:
                slugify_filename = slugify(item.filename)

                if len(slugify_filename) <= 4:
                    context.log.warning(
                        f"Unable to parse filename {item.filename}"
                    )
                    continue

                if slugify_filename[-4] != "-":
                    context.log.warning(
                        f"Unable to parse extension for {item.filename}"
                    )
                    continue

                object_filename = ".".join(
                    [slugify_filename[:-4], slugify_filename[-4:][1:]]
                )

                if partitioned:
                    current_partition = get_current_partition()
                    object_name = f"{destination}/{current_partition}/{path}/{object_filename}"  # noqa: E501
                else:
                    object_name = f"{destination}/{path}/{object_filename}"

                object_name = object_name.replace("./", "").lower()

                raw_data = file_resource.read()
                file_hash = sha512(raw_data).hexdigest()

                partition_key, metadata = prepare_object(
                    file_resource,
                    partitioned,
                )
                metadata["hash"] = file_hash

                file_to_append = {
                    "name": object_name,
                    "filename": item.filename,
                    "hash": file_hash,
                    "partition_key": partition_key,
                    "is_new": False,
                }

                object_stat = context.resources.legacy_objects.stat(object_name)
                if object_stat:
                    context.log.debug(f"{object_name} already exists")
                    # Update object or create a new version

                    if (
                        metadata["hash"]
                        == object_stat.metadata["x-amz-meta-hash"]
                    ):
                        context.log.debug(
                            f"{item.filename} and {object_name} seems to be the same"  # noqa: E501
                        )
                        files.append(file_to_append)
                        continue

                context.log.info(
                    f"Upload {item.filename} to s3 as {object_name}..."
                )

                context.resources.legacy_objects.write(
                    object_name,
                    raw_data,
                    "text/csv",
                    metadata,
                )
                file_to_append["is_new"] = True
                files.append(file_to_append)
# fmt: on
